Method: Concurrent.dataflow
- Defined in:
- lib/concurrent-ruby/concurrent/dataflow.rb
.dataflow(*inputs) {|inputs| ... } ⇒ Object
Dataflow allows you to create a task that will be scheduled when all of its data dependencies are available.
34 35 36 |
# File 'lib/concurrent-ruby/concurrent/dataflow.rb', line 34 def dataflow(*inputs, &block) dataflow_with(Concurrent.global_io_executor, *inputs, &block) end |