Class: Concurrent::AtomicBoolean

Inherits:
AtomicBooleanImplementation
  • Object
show all
Defined in:
lib/concurrent/atomic/atomic_boolean.rb

Overview

A boolean value that can be updated atomically. Reads and writes to an atomic boolean and thread-safe and guaranteed to succeed. Reads and writes may block briefly but no explicit locking is required.

Testing with ruby 2.1.2 Testing with Concurrent::MutexAtomicBoolean... 2.790000 0.000000 2.790000 ( 2.791454) Testing with Concurrent::CAtomicBoolean... 0.740000 0.000000 0.740000 ( 0.740206)

Testing with jruby 1.9.3 Testing with Concurrent::MutexAtomicBoolean... 5.240000 2.520000 7.760000 ( 3.683000) Testing with Concurrent::JavaAtomicBoolean... 3.340000 0.010000 3.350000 ( 0.855000)

Instance Method Summary collapse

Constructor Details

#initialize(initial = false) ⇒ Object

Creates a new AtomicBoolean with the given initial value.

Parameters:

  • initial (Boolean) (defaults to: false)

    the initial value



114
115
# File 'lib/concurrent/atomic/atomic_boolean.rb', line 114

class AtomicBoolean < AtomicBooleanImplementation
end

Instance Method Details

#false?Boolean

Is the current value false

Returns:

  • (Boolean)

    true if the current value is false, else false



114
115
# File 'lib/concurrent/atomic/atomic_boolean.rb', line 114

class AtomicBoolean < AtomicBooleanImplementation
end

#make_falseBoolean

Explicitly sets the value to false.

Returns:

  • (Boolean)

    true is value has changed, otherwise false



114
115
# File 'lib/concurrent/atomic/atomic_boolean.rb', line 114

class AtomicBoolean < AtomicBooleanImplementation
end

#make_trueBoolean

Explicitly sets the value to true.

Returns:

  • (Boolean)

    true is value has changed, otherwise false



114
115
# File 'lib/concurrent/atomic/atomic_boolean.rb', line 114

class AtomicBoolean < AtomicBooleanImplementation
end

#true?Boolean

Is the current value true

Returns:

  • (Boolean)

    true if the current value is true, else false



114
115
# File 'lib/concurrent/atomic/atomic_boolean.rb', line 114

class AtomicBoolean < AtomicBooleanImplementation
end

#valueBoolean

Retrieves the current Boolean value.

Returns:

  • (Boolean)

    the current value



114
115
# File 'lib/concurrent/atomic/atomic_boolean.rb', line 114

class AtomicBoolean < AtomicBooleanImplementation
end

#value=(value) ⇒ Boolean

Explicitly sets the value.

Parameters:

  • value (Boolean)

    the new value to be set

Returns:

  • (Boolean)

    the current value



114
115
# File 'lib/concurrent/atomic/atomic_boolean.rb', line 114

class AtomicBoolean < AtomicBooleanImplementation
end