Class: Concurrent::ImmediateExecutor

Inherits:
Object
  • Object
show all
Defined in:
lib/concurrent/executor/immediate_executor.rb

Overview

Note:

Intended for use primarily in testing and debugging.

An executor service which runs all operations on the current thread, blocking as necessary. Operations are performed in the order they are received and no two operations can be performed simultaneously.

This executor service exists mainly for testing an debugging. When used it immediately runs every #post operation on the current thread, blocking that thread until the operation is complete. This can be very beneficial during testing because it makes all operations deterministic.

Direct Known Subclasses

IndirectImmediateExecutor

Instance Method Summary collapse

Constructor Details

#initializeImmediateExecutor

Creates a new executor



20
21
22
# File 'lib/concurrent/executor/immediate_executor.rb', line 20

def initialize
  @stopped = Concurrent::Event.new
end

Instance Method Details

#<<(task) ⇒ self

Submit a task to the executor for asynchronous processing.

Parameters:

  • task (Proc)

    the asynchronous task to perform

Returns:

  • (self)

    returns itself



33
34
35
36
# File 'lib/concurrent/executor/immediate_executor.rb', line 33

def <<(task)
  post(&task)
  self
end

#post(*args) { ... } ⇒ Boolean

Submit a task to the executor for asynchronous processing.

Parameters:

  • args (Array)

    zero or more arguments to be passed to the task

Yields:

  • the asynchronous task to perform

Returns:

  • (Boolean)

    true if the task is queued, false if the executor is not running

Raises:

  • (ArgumentError)

    if no task is given



25
26
27
28
29
30
# File 'lib/concurrent/executor/immediate_executor.rb', line 25

def post(*args, &task)
  raise ArgumentError.new('no block given') unless block_given?
  return false unless running?
  task.call(*args)
  true
end

#running?Boolean

Returns:

  • (Boolean)


39
40
41
# File 'lib/concurrent/executor/immediate_executor.rb', line 39

def running?
  ! shutdown?
end

#shutdownObject Also known as: kill



54
55
56
57
# File 'lib/concurrent/executor/immediate_executor.rb', line 54

def shutdown
  @stopped.set
  true
end

#shutdown?Boolean

Returns:

  • (Boolean)


49
50
51
# File 'lib/concurrent/executor/immediate_executor.rb', line 49

def shutdown?
  @stopped.set?
end

#shuttingdown?Boolean

Returns:

  • (Boolean)


44
45
46
# File 'lib/concurrent/executor/immediate_executor.rb', line 44

def shuttingdown?
  false
end

#wait_for_termination(timeout = nil) ⇒ Object



61
62
63
# File 'lib/concurrent/executor/immediate_executor.rb', line 61

def wait_for_termination(timeout = nil)
  @stopped.wait(timeout)
end