Class: Concurrent::SimpleExecutorService

Inherits:
RubyExecutorService show all
Defined in:
lib/concurrent/executor/simple_executor_service.rb

Overview

Note:

Intended for use primarily in testing and debugging.

An executor service in which every operation spawns a new, independently operating thread.

This is perhaps the most inefficient executor service in this library. It exists mainly for testing an debugging. Thread creation and management is expensive in Ruby and this executor performs no resource pooling. This can be very beneficial during testing and debugging because it decouples the using code from the underlying executor implementation. In production this executor will likely lead to suboptimal performance.

Direct Known Subclasses

PerThreadExecutor

Constant Summary

Constants inherited from AbstractExecutorService

AbstractExecutorService::FALLBACK_POLICIES

Class Method Summary collapse

Instance Method Summary collapse

Methods inherited from Concurrent::Synchronization::Object

#new_condition

Class Method Details

.<<(task) ⇒ self

Submit a task to the executor for asynchronous processing.

Parameters:

  • task (Proc)

    the asynchronous task to perform

Returns:

  • (self)

    returns itself



31
32
33
34
# File 'lib/concurrent/executor/simple_executor_service.rb', line 31

def self.<<(task)
  post(&task)
  self
end

.post(*args) { ... } ⇒ Boolean

Submit a task to the executor for asynchronous processing.

Parameters:

  • args (Array)

    zero or more arguments to be passed to the task

Yields:

  • the asynchronous task to perform

Returns:

  • (Boolean)

    true if the task is queued, false if the executor is not running

Raises:

  • (ArgumentError)

    if no task is given



21
22
23
24
25
26
27
28
# File 'lib/concurrent/executor/simple_executor_service.rb', line 21

def self.post(*args)
  raise ArgumentError.new('no block given') unless block_given?
  Thread.new(*args) do
    Thread.current.abort_on_exception = false
    yield(*args)
  end
  true
end

Instance Method Details

#<<(task) ⇒ self

Submit a task to the executor for asynchronous processing.

Parameters:

  • task (Proc)

    the asynchronous task to perform

Returns:

  • (self)

    returns itself



53
54
55
56
# File 'lib/concurrent/executor/simple_executor_service.rb', line 53

def <<(task)
  post(&task)
  self
end

#killObject



81
82
83
84
85
# File 'lib/concurrent/executor/simple_executor_service.rb', line 81

def kill
  @running.make_false
  @stopped.set
  true
end

#post(*args) { ... } ⇒ Boolean

Submit a task to the executor for asynchronous processing.

Parameters:

  • args (Array)

    zero or more arguments to be passed to the task

Yields:

  • the asynchronous task to perform

Returns:

  • (Boolean)

    true if the task is queued, false if the executor is not running

Raises:

  • (ArgumentError)

    if no task is given



37
38
39
40
41
42
43
44
45
46
47
48
49
50
# File 'lib/concurrent/executor/simple_executor_service.rb', line 37

def post(*args, &task)
  raise ArgumentError.new('no block given') unless block_given?
  return false unless running?
  @count.increment
  Thread.new(*args) do
    Thread.current.abort_on_exception = false
    begin
      yield(*args)
    ensure
      @count.decrement
      @stopped.set if @running.false? && @count.value == 0
    end
  end
end

#running?Boolean

Returns:

  • (Boolean)


59
60
61
# File 'lib/concurrent/executor/simple_executor_service.rb', line 59

def running?
  @running.true?
end

#shutdownObject



74
75
76
77
78
# File 'lib/concurrent/executor/simple_executor_service.rb', line 74

def shutdown
  @running.make_false
  @stopped.set if @count.value == 0
  true
end

#shutdown?Boolean

Returns:

  • (Boolean)


69
70
71
# File 'lib/concurrent/executor/simple_executor_service.rb', line 69

def shutdown?
  @stopped.set?
end

#shuttingdown?Boolean

Returns:

  • (Boolean)


64
65
66
# File 'lib/concurrent/executor/simple_executor_service.rb', line 64

def shuttingdown?
  @running.false? && ! @stopped.set?
end

#wait_for_termination(timeout = nil) ⇒ Object



88
89
90
# File 'lib/concurrent/executor/simple_executor_service.rb', line 88

def wait_for_termination(timeout = nil)
  @stopped.wait(timeout)
end