Class: Concurrent::ThreadPoolExecutor

Inherits:
ThreadPoolExecutorImplementation
  • Object
show all
Defined in:
lib/concurrent/executor/thread_pool_executor.rb

Overview

Note:

Failure to properly shutdown a thread pool can lead to unpredictable results. Please read *Shutting Down Thread Pools* for more information.

An abstraction composed of one or more threads and a task queue. Tasks (blocks or proc objects) are submit to the pool and added to the queue. The threads in the pool remove the tasks and execute them in the order they were received. When there are more tasks queued than there are threads to execute them the pool will create new threads, up to the configured maximum. Similarly, threads that are idle for too long will be garbage collected, down to the configured minimum options. Should a thread crash it, too, will be garbage collected.

ThreadPoolExecutor is based on the Java class of the same name. From the official Java documentationa;

> Thread pools address two different problems: they usually provide > improved performance when executing large numbers of asynchronous tasks, > due to reduced per-task invocation overhead, and they provide a means > of bounding and managing the resources, including threads, consumed > when executing a collection of tasks. Each ThreadPoolExecutor also > maintains some basic statistics, such as the number of completed tasks. > > To be useful across a wide range of contexts, this class provides many > adjustable parameters and extensibility hooks. However, programmers are > urged to use the more convenient Executors factory methods > [CachedThreadPool] (unbounded thread pool, with automatic thread reclamation), > [FixedThreadPool] (fixed size thread pool) and [SingleThreadExecutor] (single > background thread), that preconfigure settings for the most common usage > scenarios.

**Thread Pool Options**

Thread pools support several configuration options:

  • idletime: The number of seconds that a thread may be idle before being reclaimed.

  • max_queue: The maximum number of tasks that may be waiting in the work queue at any one time. When the queue size reaches max_queue subsequent tasks will be rejected in accordance with the configured fallback_policy.

  • auto_terminate: When true (default) an at_exit handler will be registered which will stop the thread pool when the application exits. See below for more information on shutting down thread pools.

  • fallback_policy: The policy defining how rejected tasks are handled.

Three fallback policies are supported:

  • :abort: Raise a RejectedExecutionError exception and discard the task.

  • :discard: Discard the task and return false.

  • :caller_runs: Execute the task on the calling thread.

**Shutting Down Thread Pools**

Killing a thread pool while tasks are still being processed, either by calling the #kill method or at application exit, will have unpredictable results. There is no way for the thread pool to know what resources are being used by the in-progress tasks. When those tasks are killed the impact on those resources cannot be predicted. The best practice is to explicitly shutdown all thread pools using the provided methods:

  • Call #shutdown to initiate an orderly termination of all in-progress tasks

  • Call #wait_for_termination with an appropriate timeout interval an allow the orderly shutdown to complete

  • Call #kill *only when* the thread pool fails to shutdown in the allotted time

On some runtime platforms (most notably the JVM) the application will not exit until all thread pools have been shutdown. To prevent applications from “hanging” on exit all thread pools include an at_exit handler that will stop the thread pool when the application exists. This handler uses a brute force method to stop the pool and makes no guarantees regarding resources being used by any tasks still running. Registration of this at_exit handler can be prevented by setting the thread pool’s constructor :auto_terminate option to false when the thread pool is created. All thread pools support this option.

“‘ruby pool1 = Concurrent::FixedThreadPool.new(5) # an at_exit handler will be registered pool2 = Concurrent::FixedThreadPool.new(5, auto_terminate: false) # prevent at_exit handler registration “`

Direct Known Subclasses

CachedThreadPool, FixedThreadPool

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#completed_task_countInteger (readonly)

The number of tasks that have been completed by the pool since construction.

Returns:

  • (Integer)

    The number of tasks that have been completed by the pool since construction.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#fallback_policyObject (readonly)



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#idletimeInteger (readonly)

The number of seconds that a thread may be idle before being reclaimed.

Returns:

  • (Integer)

    The number of seconds that a thread may be idle before being reclaimed.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#largest_lengthInteger (readonly)

The largest number of threads that have been created in the pool since construction.

Returns:

  • (Integer)

    The largest number of threads that have been created in the pool since construction.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#lengthInteger (readonly)

The number of threads currently in the pool.

Returns:

  • (Integer)

    The number of threads currently in the pool.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#max_lengthInteger (readonly)

The maximum number of threads that may be created in the pool.

Returns:

  • (Integer)

    The maximum number of threads that may be created in the pool.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#max_queueInteger (readonly)

The maximum number of tasks that may be waiting in the work queue at any one time. When the queue size reaches max_queue subsequent tasks will be rejected in accordance with the configured fallback_policy.

Returns:

  • (Integer)

    The maximum number of tasks that may be waiting in the work queue at any one time. When the queue size reaches max_queue subsequent tasks will be rejected in accordance with the configured fallback_policy.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#min_lengthInteger (readonly)

The minimum number of threads that may be retained in the pool.

Returns:

  • (Integer)

    The minimum number of threads that may be retained in the pool.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#queue_lengthInteger (readonly)

The number of tasks in the queue awaiting execution.

Returns:

  • (Integer)

    The number of tasks in the queue awaiting execution.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#remaining_capacityInteger (readonly)

Number of tasks that may be enqueued before reaching max_queue and rejecting new tasks. A value of -1 indicates that the queue may grow without bound.

Returns:

  • (Integer)

    Number of tasks that may be enqueued before reaching max_queue and rejecting new tasks. A value of -1 indicates that the queue may grow without bound.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#scheduled_task_countInteger (readonly)

The number of tasks that have been scheduled for execution on the pool since construction.

Returns:

  • (Integer)

    The number of tasks that have been scheduled for execution on the pool since construction.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

Instance Method Details

#<<(task) ⇒ self

Submit a task to the executor for asynchronous processing.

Parameters:

  • task (Proc)

    the asynchronous task to perform

Returns:

  • (self)

    returns itself



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#auto_terminate=(value) ⇒ Object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#auto_terminate?Object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#can_overflow?Boolean

Does the task queue have a maximum size?

Returns:

  • (Boolean)

    True if the task queue has a maximum size else false.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#killObject



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#post(*args) { ... } ⇒ Boolean

Submit a task to the executor for asynchronous processing.

Parameters:

  • args (Array)

    zero or more arguments to be passed to the task

Yields:

  • the asynchronous task to perform

Returns:

  • (Boolean)

    true if the task is queued, false if the executor is not running

Raises:

  • (ArgumentError)

    if no task is given



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#running?Object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#serialized?Boolean

Does this executor guarantee serialization of its operations?

Returns:

  • (Boolean)

    True if the executor guarantees that all operations will be post in the order they are received and no two operations may occur simultaneously. Else false.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#shutdownObject



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#shutdown?Object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#shuttingdown?Object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end

#wait_for_termination(timeout = nil) ⇒ Object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/concurrent/executor/thread_pool_executor.rb', line 50

class ThreadPoolExecutor < ThreadPoolExecutorImplementation

  # @!macro [new] thread_pool_executor_method_initialize
  #
  #   Create a new thread pool.
  #  
  #   @param [Hash] opts the options which configure the thread pool.
  #  
  #   @option opts [Integer] :max_threads (DEFAULT_MAX_POOL_SIZE) the maximum
  #     number of threads to be created
  #   @option opts [Integer] :min_threads (DEFAULT_MIN_POOL_SIZE) the minimum
  #     number of threads to be retained
  #   @option opts [Integer] :idletime (DEFAULT_THREAD_IDLETIMEOUT) the maximum
  #     number of seconds a thread may be idle before being reclaimed
  #   @option opts [Integer] :max_queue (DEFAULT_MAX_QUEUE_SIZE) the maximum
  #     number of tasks allowed in the work queue at any one time; a value of
  #     zero means the queue may grow without bound
  #   @option opts [Symbol] :fallback_policy (:abort) the policy for handling new
  #     tasks that are received when the queue size has reached
  #     `max_queue` or the executor has shut down
  #  
  #   @raise [ArgumentError] if `:max_threads` is less than one
  #   @raise [ArgumentError] if `:min_threads` is less than zero
  #   @raise [ArgumentError] if `:fallback_policy` is not one of the values specified
  #     in `FALLBACK_POLICIES`
  #  
  #   @see http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ThreadPoolExecutor.html

  # @!method initialize(opts = {})
  #   @!macro thread_pool_executor_method_initialize
end