Create a new thread pool.
Options Hash (opts):
the policy for handling new tasks that are received when the queue size has reached max_queue or the executor has shut down
max_queue
Raises:
if :fallback_policy is not one of the values specified in FALLBACK_POLICIES
:fallback_policy
FALLBACK_POLICIES
See Also:
# File 'lib/concurrent/exchanger.rb', line 337