Method: Concurrent.dataflow

Defined in:
lib/concurrent/dataflow.rb

.dataflow(*inputs) {|inputs| ... } ⇒ Object

Parameters:

  • inputs (Future)

    zero or more Future operations that this dataflow depends upon

Yields:

  • The operation to perform once all the dependencies are met

Yield Parameters:

  • inputs (Future)

    each of the Future inputs to the dataflow

Yield Returns:

  • (Object)

    the result of the block operation

Returns:

  • (Object)

    the result of all the operations

Raises:

  • (ArgumentError)

    if no block is given

  • (ArgumentError)

    if any of the inputs are not ‘IVar`s



34
35
36
# File 'lib/concurrent/dataflow.rb', line 34

def dataflow(*inputs, &block)
  dataflow_with(Concurrent.configuration.global_operation_pool, *inputs, &block)
end