Method: Concurrent::MutableStruct#values_at
- Defined in:
- lib/concurrent/mutable_struct.rb
#values_at(*indexes) ⇒ Object
Returns the struct member values for each selector as an Array.
A selector may be either an Integer offset or a Range of offsets (as in ‘Array#values_at`).
63 64 65 |
# File 'lib/concurrent/mutable_struct.rb', line 63 def values_at(*indexes) synchronize { ns_values_at(indexes) } end |