Method: Concurrent::SettableStruct#values_at
- Defined in:
- lib/concurrent/settable_struct.rb
#values_at(*indexes) ⇒ Object
Returns the struct member values for each selector as an Array.
A selector may be either an Integer offset or a Range of offsets (as in Array#values_at).
24 25 26 |
# File 'lib/concurrent/settable_struct.rb', line 24 def values_at(*indexes) synchronize { ns_values_at(indexes) } end |