Class: Concurrent::ThreadLocalVar
- Inherits:
-
ThreadLocalVarImplementation
- Object
- Concurrent::ThreadLocalVar
- Defined in:
- lib/concurrent/atomic/thread_local_var.rb
Overview
A ThreadLocalVar is a variable where the value is different for each thread. Each variable may have a default value, but when you modify the variable only the current thread will ever see that change.